body,
p, div, ol, li,
form, input
img {
    margin: 0;
    padding: 0;
}

* {
    font-family: Arial, sans-serif;
}

body {
    background-color: #fcfcfc;
    color: #404040;
    font-size: 100%; /* 16px */
}

h2 {
    padding-left: 1.5ex;
}

p {
    margin-bottom: 1em;
}

a {
    text-decoration: none;
    color: #2980b9;
}

.header {
    background-color: #2980b9;
    color: #fcfcfc;
}

.jump-form {
    float: right;
}

.search-form, .jump-form {
    padding: 1em;
}

.search-form input, .search-form select, .jump-form select {
    font-size: 1.1em;
}

.doc-sets li {
    list-style-type: none;
    font-size: 1.1em;
    padding-bottom: 0.5em;
}

.hits {
    padding: 1em;
}

.hits li {
    margin-left: 1em;
    padding-bottom: 1em;
}

.document-kind, .docset {
    color: #999;
}
